#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <stdbool.h>

#define type_elem int
//si type_elem est modifié, affiche_elem doit être adaptée


struct s_cellule{
	type_elem val; //valeur de l'élément
	struct s_cellule* next; //adresse de la cellule suivante
	//ou NULL si c'est la dernière cellule de la liste
};

typedef struct s_cellule cellule;
typedef cellule* liste_c;


cellule* create_cell(type_elem elem){
	//retourne un pointeur vers une cellule 
	//contenant elem et pointant vers NULL
	//attention allocation de mémoire dynamique
	//penser à libérer le pointeur retourné après usage	
	cellule* res = (cellule*) malloc(sizeof(cellule));
	(*res).val = elem;
	(*res).next = NULL;
	return res;
}



void affiche_elem(type_elem elem){
	//affiche elem
	printf("%d",elem);
}

void affiche_liste(liste_c l){
	//hyp : l n'est pas NULL
	//affiche les éléments de la liste l
	cellule* p = l;
	//afficher l'elem de la 1ere cellule
	affiche_elem(p->val);
	printf("  ");
	//tant qu'il y a une cellule suivante, 
	while(p->next != NULL){
		//s'y déplacer 
		p = p->next;
		//et afficher son elem
		affiche_elem(p->val);
		printf("  ");
	}
};



int nb_elem(liste_c l){
	//hyp : l n'est pas NULL
	//retourne le nombre d'élém de l
	//c'est aussi son nb de cellules
	cellule* p = l;
	int res = 1;
	while(p->next != NULL){
		p = p->next;
		res = res + 1;
	}
	return res;
}



liste_c create_from_tab(type_elem* tab, int lg){
	// tab est de taille lg>0
	// retourne une liste chainée contenant les éléments de tab
	// attention allocation dynamique de mémoire
	// utiliser free_liste_c sur le pointeur retourné après usage
	
	cellule* res = create_cell(tab[0]);
	cellule* prec = res;
	int i = 1;
	while(i < lg){
		prec->next = create_cell(tab[i]);
		prec = prec->next;
		i = i + 1;
	}
	return res;
}



void free_liste_c(liste_c l){
	//hyp : l n'est pas NULL
	//libère la mémoire occupée par les cellules de l
	cellule* p = l;
	cellule* temp;
	while(p->next != NULL){
		temp = p->next;
		free(p);
		p = temp;
	}
}



bool contient_elem(type_elem x, liste_c l){
	//hyp : l n'est pas NULL
	//retourne si x est un élément de la liste l
	cellule* p = l;
	while(true){
		if(x == p->val)
			return true;
		else if (p->next == NULL)
			return false;
		else
			p = p->next;
	}
};



void ajoute_elem_debut(type_elem x, liste_c* ptr_l){
	//met à jour la liste *ptr_l passée par référence
	//pour que ce soit la liste l devant laquelle on a ajouté x
	cellule* res = create_cell(x);
	res->next = (*ptr_l);
	*ptr_l = res;
}

